var ShakeShackWidget = {

	CONFIG : {
		camera_url : "http://shakeshacknyc.com/camera.jpg",
		calendar_url : "http://widgets.worksperfectly.net/shakeshack/data/calendar.xml?date="
	},

	DOM : {
		widget_front : null,
		widget_back : null,
		info_button : null,
		hide_button : null
	},

	xhr : new XMLHttpRequest(),
	image_loading_timeout : null,

	// -------------------------------------------------------------------------


	init : function() {
		this.DOM.widget_front = document.getElementById("WidgetFront");
		this.DOM.widget_back = document.getElementById("WidgetBack");
		this.DOM.info_button = new AppleInfoButton(document.getElementById("info-button"), this.DOM.widget_front, "white", "white", function() { ShakeShackWidget.toggle(); });
		this.DOM.done_button = new AppleGlassButton(document.getElementById("done-button"), "Done", function() { ShakeShackWidget.toggle(); });

		document.getElementById('shackcam').onload = function() {
			this.style.display = "block";

			var now = new Date();

			var hours = now.getHours();
			hours = (hours > 12) ? hours - 12 : hours;
			var minutes = now.getMinutes();
			minutes = (minutes < 10) ? "0" + minutes : minutes;
			var seconds = now.getSeconds();
			seconds = (seconds < 10) ? "0" + seconds : seconds;

			document.getElementById('last-updated-time').innerHTML = "" + hours + ":" + minutes + ":" + seconds + " " + ((now.getHours() >= 12) ? "PM" : "AM");
			document.getElementById('camera-frame').style.display = "block";
			document.getElementById('last-updated').style.display = "block";

			var current_hour = now.getHours();
			var open_close_msg = "";
			var status_class = "";
			var target_hour;
			var target_action = "";

			// if it's really late, display the closed message
			if ( (current_hour >= 23) || (current_hour < 6) ) {
				open_close_msg = 'Shake Shack is closed.';
				status_class = "closed";

			// otherwise, display something if near opening or closing times
			} else {

				// closed, but near opening time
				if ( (current_hour < 11) && (current_hour > 5) ) {
					target_hour = 11;
					status_class = "opening";
					target_action = "opens";

				// close to closing
				} else if (current_hour >= 20) {
					target_hour = 23;
					status_class = "closing";
					target_action = "closes";

				// else, just hide the damn thing!
				} else {
					target_hour = -1;
				}

				var time = new Date(now.getFullYear(), now.getMonth(),now.getDate(), target_hour, 00, 00, 00);
				var seconds_til = (time - now) / 1000;
				var hours_til = Math.floor(seconds_til / 60 / 60);
				var minutes_til = Math.floor(seconds_til / 60) - (hours_til * 60);
				var time_string = ((hours_til == 0) ? "" : hours_til + " hour" + (hours_til == 1 ? "" : "s")) + " " + minutes_til + " minute" + (minutes_til == 1 ? "" : "s");

				open_close_msg = (target_hour < 0) ? "" : time_string + " until Shake Shack " + target_action;
			}

			document.getElementById('shack-status').innerHTML = open_close_msg;
			document.getElementById('shack-status').className = status_class;
			document.getElementById('shack-status').style.visibility = "visible";

		} // end image onload handler

		if (!window.widget) {
			this.loadCalendarData();
			this.refreshCamera();
		}

	}, // END: init


	loadCalendarData : function() {
		this.xhr.open("GET", this.CONFIG.calendar_url + this.getDateAsString(), true);
		this.xhr.onreadystatechange = this.displayCalendarData;
		this.xhr.send(null);
	}, // END: loadCalendarData


	displayCalendarData : function() {
		var self = ShakeShackWidget;
		if (self.xhr.readyState != 4) { return; }

		var date_string = self.getDateAsString();
		var custard = "";
		var note = "";

		try {
			alert(self.xhr.responseText);

			var day_nodes = self.xhr.responseXML.getElementsByTagName('day');
			for (var i=0; i < day_nodes.length; i++) {
				if (day_nodes[i].getAttribute('date') != date_string) { continue; }

				var custards = day_nodes[i].getElementsByTagName('custard');
				if (custards.length) {
					custard = custards[0].childNodes[0].nodeValue;
				}

				var notes = day_nodes[i].getElementsByTagName('note');
				if (notes.length) {
					note = notes[0].childNodes[0].nodeValue;
				}

				break;
			}
		} catch(e) { }

		if (note != "") {
			//  do something with the note
		}

		// set the custard flavor and/or fail out silently if not custard exists
		document.getElementById('custard-flavor').innerHTML = custard;
		document.getElementById('todays-custard').style.display = (custard == "") ? "none" : "block";
		
	}, // END: displayCalendarData


	getDateAsString : function() {
		var now = new Date();
		var year = now.getFullYear();
		var month = now.getMonth();
		month++;
		month = (month < 10) ? "0" + month : month;
		var day = now.getDate();
		day = (day < 10) ? "0" + day : day;

		return (year + "-" + month + "-" + day);
	}, // END: getDateAsString


	refreshCamera : function() {
		document.getElementById('shackcam').src = this.CONFIG.camera_url + "?" + Math.floor(Math.random() * 10000000);
		ShakeShackWidget.image_loading_timeout = setTimeout(function(){ ShakeShackWidget.refreshCamera()}, 5000);
	}, // END: refreshCamera


	toggle : function() {

		if (window.widget) {
			widget.prepareForTransition((this.DOM.widget_front.style.display == "block") ? "ToBack" : "ToFront");
		}

		this.DOM.widget_front.style.display = (this.DOM.widget_front.style.display == "block") ? "none" : "block";
		this.DOM.widget_back.style.display = (this.DOM.widget_back.style.display == "none") ? "block" : "none";

		if (window.widget) {
			setTimeout("widget.performTransition()", 0);
		}

	}, // END: toggle


	widgetShow : function() {
		this.loadCalendarData();
		this.refreshCamera();
		document.getElementById("loading").style.backgroundImage = "url(./images/spinner.gif)";
	}, // END: widgetShow


	widgetHide : function() {
		document.getElementById("loading").style.backgroundImage = "none"; // hide the spinner to reduce CPU usage
		clearTimeout(this.image_loading_timeout);
	} // END: widgetHide


}; // END: ShakeShackWidget


// ===========================================================================

window.onload = function() { ShakeShackWidget.init(); };

if (window.widget) {
	widget.onshow = function() { ShakeShackWidget.widgetShow(); };
	widget.onhide = function() { ShakeShackWidget.widgetHide(); };
}
